{ Encryption Common / (C) 2006-2012, Andreas Berg. All Rights Reserved. }
unit _ecommon;

interface

var
  IsRunEnc         : boolean;

  eBuf             : array[0..1024] of byte;
  maskBuf          : array[0..29] of word;
  cMask            : word;
  EPattern         : array[0..65] of string;

  sfile, efile     : string;
  sfile2           : string;
  pwd              : string;
  slen             : int64;
  pos              : int64;
  f                : file of byte;
  perc             : integer;
  st               : int64;
  newFile          : string;
  hdr              : string;
  ndir             : string;
  fff              : file of byte;
  byt              : array[1..10] of byte;

  CmdPwd           : string;
  CmdIFile         : string;
  CmdOFile         : string;



function RemoveExt(filename: string): string;
function GetFileLen(fname: string): int64;

procedure ResetBytes;
procedure ValueToBytes(v: int64);
function BytesToValue(by1,by2,by3,by4: byte): int64;

procedure InitMasks;

function IncVal(val, cnt: byte): byte;
function DecVal(val, cnt: byte): byte;

function GetEncCode(val: byte; p: integer; mask: word): byte;
function GetDecCode(val: byte; p: integer; mask: word): byte;

procedure DoEncrypt(len: word; useMask: word);
procedure DoDecrypt(len: word; useMask: word);

procedure WriteByte(b: byte);
procedure WriteWord(w: word);
procedure WriteString(s: string);
procedure WriteRAWStr(s: string);

function ReadByte: byte;
function ReadWord: word;
function ReadString: string;
function ReadStringN(ln: integer): string;

implementation

function RemoveExt(filename: string): string;
var
  pos              : integer;
  newPath          : string;
begin
   pos:= Length(filename);
   newPath:= filename;
   if pos > 0 then begin
      while true do begin
         if pos = 1 then break;
         if filename[pos] = '.' then begin
            Dec(pos);
            if pos > 0 then newPath:= Copy(filename, 1, pos);
            break;
         end;
         Dec(pos);
      end;
   end;
   RemoveExt:= newPath;
end;

function GetFileLen(fname: string): int64;
var
 f                 : file of byte;
 l                 : int64;
begin
   {$I-}
   Assign(f, fname);
   Reset(f);
      l:= FileSize(f);
   Close(f);
   {$I+}
   GetFileLen:= l;
end;



procedure ResetBytes;
var
 i                 : integer;
begin
   for i:= 1 to 10 do byt[i]:= 0;
end;

procedure ValueToBytes(v: int64);
var
 vv                : int64;
begin
   vv:= v;
   ResetBytes;
   if vv >= 0 then begin
      {byt[7]:= vv div 281474976710656;
      vv:= vv - (byt[7] * 281474976710656);
      byt[6]:= vv div 1099511627776;
      vv:= vv - (byt[6] * 1099511627776);
      byt[5]:= vv div 4294967296;
      vv:= vv - (byt[5] * 4294967296); }
      byt[4]:= vv div 16777216;
      vv:= vv - (byt[4] * 16777216);
      byt[3]:= vv div 65536;
      vv:= vv - (byt[3] * 65536);
      byt[2]:= vv div 256;
      vv:= vv - (byt[2] * 256);
      byt[1]:= vv;
   end else begin
      vv:= -1 - v;

      byt[10]:= 255;
      byt[9]:= 255;
      byt[8]:= 255;

      {byt[7]:= vv div 281474976710656;
      vv:= vv - (byt[7] * 281474976710656);
      byt[7]:= 255 - byt[7];
      byt[6]:= vv div 1099511627776;
      vv:= vv - (byt[6] * 1099511627776);
      byt[6]:= 255 - byt[6];
      byt[5]:= vv div 4294967296;
      vv:= vv - (byt[5] * 4294967296);
      byt[5]:= 255 - byt[5]; }
      byt[4]:= vv div 16777216;
      vv:= vv - (byt[4] * 16777216);
      byt[4]:= 255 - byt[4];
      byt[3]:= vv div 65536;
      vv:= vv - (byt[3] * 65536);
      byt[3]:= 255 - byt[3];
      byt[2]:= vv div 256;
      vv:= vv - (byt[2] * 256);
      byt[2]:= 255 - byt[2];
      byt[1]:= 255 - vv;
   end;
end;

function BytesToValue(by1,by2,by3,by4: byte): int64;
begin
   BytesToValue:= (by4 * 16777216) + (by3 * 65536) + (by2 * 256) + by1;
end;



procedure InitMasks;
begin
   EPattern[0] := 'ADBCHIKLIJHARTTSADBCHIKLIJHARTTSADBCHIKLIJHARTTSADBCHIKLIJHARTTS';
   EPattern[1] := 'ZXYHJOKHFRTGNJHMBVCDFGETgdijdhetgsjkducbBGFDRETTRREEEDFCFFVSSSSS';
   EPattern[2] := 'ijdiejehfkvmndheUJHSTGgdfdTG012383438TREBBSVVFFSRRRFSKKKJJUUIuud';
   EPattern[3] := 'afijuujdgttHHHHSyjdhyrehfyflijshshtrsbcbcbkhhhddeqertioGFDVBB224';
   EPattern[4] := 'AAUHGREDDDFGGBVCSSFOIUYJHFGteg35687999877IJJJjdhdhtRDFFFDdDFfssw';
   EPattern[5] := 'fdTG012383heUJHSTGgdfdTG012383heUJHSTGgdIJJJjdhdhtyyujHHJJgfsfeR';
   EPattern[6] := 'vmndheUJHKLIJHARTKLIJHARTKLIJHARTS2437874UUUHJjdhdhueyhdyhdjdhdT';
   EPattern[7] := 'heUJH83heUJHSTGgd83heUJHSTGgdSTGgdfdTG01IJJJjdhdhtpoijhdgfdfdR23';
   EPattern[8] := 'KLIJHARTKLIJHARnncvffgdtfdjuUJHGDDDGGGtttegggdsggvnmkjhffyuiresT';
   EPattern[9] := 'IJJJjdKLIJHARThdhtRyuyuyugfsfsfrrw45668ijuydgr545fsfgfdrtrfggssk';
   EPattern[10]:= 'KLIJHARTpouyryHHHSJJJHooiooioi437ujncjdudjdjdj87f7fjs4566s544sss';
   EPattern[11]:= 'fdTG012heUJH83heUJHSTGgdSTGgdfdTG0123383vmndheUJHSsiuiiiuunncccT';
   EPattern[12]:= 'heUJHHSSFOIUYJHFGSSFOIUYJHFGAndh83hIRTS24ndhndh83hI83hiiurrdI3ST';
   EPattern[13]:= 'KLIJHARHSSFOIUYJHSSFOIUYJHFGFndh83hIGARndh83hITSndh83hI243eerssT';
   EPattern[14]:= 'heUJHSSFOIUYJHFGSSFOIUYJHFGSndh83hIndh83hIToi0987756635363622344';
   EPattern[15]:= '83heUJKLSSFOIUYJHFGIJHARTHSTGgd83heUJHSTGgd83HARTS243heUJHSTGgd4';
   EPattern[16]:= 'heUJHHARTSSSFOIUYJHFGSSFOIUYJHFG24ndh83hIndh83hI3ST55788733yuudd';
   EPattern[17]:= 'heUJheHARTS243UJHSTheUJHSTHSndh83hIndh8uhfnmvhjjeduufiisdhjh3hIT';
   EPattern[18]:= 'heUJHHASSFOIUYJHFGSSFOIUYJHFGRTS2ndh83hIndh83hI43iooiiiidddddsST';
   EPattern[19]:= 'vmndheUJIKLIJHARTJJJjdhdhtRheUJHSTG83heUJHSTGgdgdfdTG0123HSTfdTG';
   EPattern[20]:= 'IJJJjdhdKLIJHARThndh83hItRabgggdfTTFDDIKJKUJiIiIiIkjjjuiuuej7783';
   EPattern[21]:= 'SSFOIUYJHFGSSFOIUYJHFGndh8OIUYJHFGAnd3hISSFOIUYJHFGiuihjjjhddvvn';
   EPattern[22]:= 'vmndheUheUJHSTGgdfndh83hOIUYJHFGAndIndh83hIdTG0123JHST3777d88d9d';
   EPattern[23]:= 'ndh83hIndST83heUOIUYJHFGOIUYJHFGAndAndJHh83h23778dhhd8hcnuduUHSI';
   EPattern[24]:= 'IJJJjdhdKLST83heUJHIJHAROIUYJHFGAndTKLIJHARTKLIJHAuHHSTRRScRThtR';
   EPattern[25]:= 'heUJST83heUJHOIUYJdvheOHFGAOIUYJHFGAndndHSTpoiIIUUi86trrbbvvzxxc';
   EPattern[26]:= 'UYJHFGSndST83OIUYJHFGAOIUYJHFGAndndheUJHhZOrrreeiiudhhhHH6653433';
   EPattern[27]:= 'vmndh83hIJJJjdhdhtReUJHSTGgdheUJHSTGgdfdTG0123eUJHSTplmkjuhdgfre';
   EPattern[28]:= 'UYJHFGSnST83heUJOdvheOIUYJHFGAndHuyyyetBGGFFDEESssfs4353ff5s6sdh';
   EPattern[29]:= 'UYJHFST83heUJHGOIdvheOUYJHFGAnheOHFGAOIUYJHFGAndndHSTpoidSndh874';
   EPattern[30]:= 'fdTG012KLIJHARTKLIJHARTKLIJHAheOHFGAOIUYJHFGAndndHSTpoiRT383yyru';
   EPattern[31]:= 'UYJHFGOIUYJHFGAndOIUYJHFGAndSndhheOHFGAOIUYJHFGAndndHSTpoccccdei';
   EPattern[32]:= 'heUJHSTGgdfOIUYJHFGAndOIUYJHFGAnddTG0123uudhhhjhdttggdfffAVVCDRE';
   EPattern[33]:= 'heUJHST83heUJHSTGgdGgdfdTheOHFGAOIUYJHFGAndndHSTpoiG012yZTTZXVC3';
   EPattern[34]:= 'UYJHFGSnTGgdvheUJHSTGgdfdhheOHFGAOIUYJHFGAndndHSTpoiookdjjjYYUHS';
   EPattern[35]:= 'vmndheUJ83heUJHSTGgddvheOHheUJHSTGgdfdTG0123STwwwandreasbergsess';
   EPattern[36]:= 'heUJHIJJJjKLIJHARTKLIJHARTdhdhtRSTG83heUJHSTGgdgdfdTG0123dotcoms';
   EPattern[37]:= 'fdTG0123h83heUJHSTGdvheOgdeUJHSTGgdfdTG012383bergoshasntstarteds';
   EPattern[38]:= 'UYJHFGSTGgdvheUJHSTGgdfheOHFGAOIUYJHFGAndndHSTpoindhheheIwannafu';
   EPattern[39]:= 'KLIJHTGgdvheUJHSTGgdvheOdheOHFGAOIUYJHFGAndndHSTpoifARTckyouinth';
   EPattern[40]:= '83heUKLIJHARTKLIJHARTJHSTheOHFGAOIUYJHFGAndndHSTpoiGgdeassdumdum';
   EPattern[41]:= 'UYJHFGSndhTGgdvheUJHheOHFGAOIUYJHFGAndndHSTpoiSTGgdfIwannasssuus';
   EPattern[42]:= 'UYJHFGSndhTGgdvheOIUYJHFGAndUJHSTGgdf4788832777899uujhtbbvcvfGFD';
   EPattern[43]:= 'vmndheheUJHST83heUJHSTGgdGgdfdTG0123UJHSTbiruuudhYHGFDDSTWQZZAAS';
   EPattern[44]:= 'KLIJHARTTGgdvheUOIUYJHFGAndJHSTGgdfbergkingnnNNNuyydTTG5663599ee';
   EPattern[45]:= 'UYJHFGSnTGgdvheUJHSTGgdvheOdvheOdfdhiuynnnmnvhhfhfjhjjrhHHHSJTTT';
   EPattern[46]:= 'vmnKLIJHARTdheheUJHSTGgdfdTG0123UJHSTfdTG012383UUIIUGGRFVVSFFGSG';
   EPattern[47]:= 'IJJJjdhdKvTGgdvheUJHSTGgdfLIJHARThtROOOIIINHNYJJFDFRTCVVBBBVCVSF';
   EPattern[48]:= 'UYJHFGSTGgdvheUJHSTGgheOHFGAOIUYJHFGAndndHSTpoidfndhuuyueuuHHJJH';
   EPattern[49]:= 'UYJHFGTGgdvheUJHOIUYJHFGAheOHFGAOIUYJHFGAndndHSTpoindSTGgdfSndhW';
   EPattern[50]:= '83heUJTGgdvheUJHOIUYJHFGAndSTGgdfHSTGgdWindowssucksass8987874ddd';
   EPattern[51]:= 'heUJHTGgdvheUJHSTGgdfSheOHFGAOIUYJHFGAndndHSTpoiT88BillssjjuUYYW';
   EPattern[52]:= 'UYJHFGSndTGgdvheOIUYJHFGAndUJHSTGgdfhmnchhy87yukionsjsjshTRRSius';
   EPattern[53]:= 'vmnKLIJHARTdheUJOIUYJHFGAndHSTGgdfdTG0123heUdmnheUJHSJHSTG0HSTab';
   EPattern[54]:= 'UYJHFGSndTGgdvheUJHSTGgdheOHFGAOIUYJHFGAndndHdmnheUJHSSTpoifhcde';
   EPattern[55]:= 'IJJJjdhdhTGgdvheOIUYJHFGAndUheOHFGAOIUYJHFGAndndHSTpoiJHSTGgdftR';
   EPattern[56]:= 'UYJHFGSndTGgdvOIUYJHFGAndheUJHSTdmnheUJHdmnheUJHSSGgHSTG0dfhfghi';
   EPattern[57]:= '83heUJHSTGgdfdTvOIUYJHFGAndmnheUJHSTGgdfdTG0123dheUJHSTG012383JU';
   EPattern[58]:= 'UYJHFGSnTGgdvheOIUYJHFGAndUJHSTGgdfdmnheUJHSHSTG0dHSTHSTdmnhG0h9';
   EPattern[59]:= 'heUJHSTGgdfdTG0TGgdvheUJHSTGgdheOHFGAOIUYJHFGAndndHSTpoif1239870';
   EPattern[60]:= 'KLIJHAheUJHSTheTOIUYJHFGAndGgdvheUJHSTdmnheUJHSGgdfUJHSTRT002973';
   EPattern[61]:= 'fdTG083heUJHSTGgdvheUJHSTGgdfdTG0123mndheUJHST123HSTdmnh83hyh773';
   EPattern[62]:= 'vmndhe8IJJJjdhKLIJHARTdhtR3heUJHSTGgdUdmnheUJHSJHHSTdmnhSee4552T';
   EPattern[63]:= 'heUJH83hIJJJjdhdhtReUJHSTGgdSTGgdfddmnheUHSTdmnhJHSTG01234444rrr';
   EPattern[64]:= 'heUJHSTGgdfdKLIJHARTTG01dmnheUJHS2HSTG0HSTHSTdmnhG0HSTG03zzzzxee';
end;



function IncVal(val, cnt: byte): byte;
var
 val2              : byte;
 t                 : integer;
begin
   t:= val + cnt;
   if t > 255 then begin
      t:= t - 256;
   end;
   val2:= t;
   IncVal:= val2;
end;

function DecVal(val, cnt: byte): byte;
var
 val2              : byte;
 t                 : integer;
begin
   t:= val - cnt;
   if t < 0 then begin
      t:= 256 + t;
   end;
   val2:= t;
   DecVal:= val2;
end;



function GetEncCode(val: byte; p: integer; mask: word): byte;
var
 met, s            : string;
 nval              : byte;
begin
   s:= EPattern[mask];
   met:= s[p+1];
   nval:= val;
   if met = 'A' then nval:= IncVal(val, 2);
   if met = 'B' then nval:= DecVal(val, 5);
   if met = 'C' then nval:= IncVal(val, 9);
   if met = 'D' then nval:= DecVal(val, 3);
   if met = 'E' then nval:= IncVal(val, 10);
   if met = 'F' then nval:= IncVal(val, 44);
   if met = 'G' then nval:= DecVal(val, 33);
   if met = 'H' then nval:= IncVal(val, 11);
   if met = 'I' then nval:= IncVal(val, 19);
   if met = 'J' then nval:= DecVal(val, 8);
   if met = 'K' then nval:= DecVal(val, 45);
   if met = 'L' then nval:= DecVal(val, 32);
   if met = 'M' then nval:= IncVal(val, 77);
   if met = 'N' then nval:= IncVal(val, 66);
   if met = 'O' then nval:= DecVal(val, 55);
   if met = 'P' then nval:= DecVal(val, 1);
   if met = 'Q' then nval:= IncVal(val, 1);
   if met = 'R' then nval:= IncVal(val, 51);
   if met = 'S' then nval:= IncVal(val, 62);
   if met = 'T' then nval:= DecVal(val, 63);
   if met = 'U' then nval:= IncVal(val, 72);
   if met = 'V' then nval:= IncVal(val, 88);
   if met = 'W' then nval:= DecVal(val, 99);
   if met = 'X' then nval:= IncVal(val, 91);
   if met = 'Y' then nval:= IncVal(val, 82);
   if met = 'Z' then nval:= IncVal(val, 85);
   if met = 'a' then nval:= DecVal(val, 92);
   if met = 'b' then nval:= IncVal(val, 94);
   if met = 'c' then nval:= IncVal(val, 96);
   if met = 'd' then nval:= DecVal(val, 83);
   if met = 'e' then nval:= IncVal(val, 38);
   if met = 'f' then nval:= IncVal(val, 37);
   if met = 'g' then nval:= DecVal(val, 36);
   if met = 'h' then nval:= DecVal(val, 24);
   if met = 'i' then nval:= DecVal(val, 25);
   if met = 'j' then nval:= DecVal(val, 26);
   if met = 'k' then nval:= DecVal(val, 27);
   if met = 'l' then nval:= DecVal(val, 87);
   if met = 'm' then nval:= IncVal(val, 8);
   if met = 'n' then nval:= IncVal(val, 9);
   if met = 'o' then nval:= DecVal(val, 13);
   if met = 'p' then nval:= IncVal(val, 16);
   if met = 'q' then nval:= DecVal(val, 19);
   if met = 'r' then nval:= IncVal(val, 21);
   if met = 's' then nval:= IncVal(val, 55);
   if met = 't' then nval:= IncVal(val, 58);
   if met = 'u' then nval:= IncVal(val, 59);
   if met = 'v' then nval:= IncVal(val, 67);
   if met = 'w' then nval:= IncVal(val, 73);
   if met = 'x' then nval:= DecVal(val, 72);
   if met = 'y' then nval:= IncVal(val, 75);
   if met = 'z' then nval:= DecVal(val, 83);
   if met = '0' then nval:= IncVal(val, 41);
   if met = '1' then nval:= IncVal(val, 46);
   if met = '2' then nval:= DecVal(val, 38);
   if met = '3' then nval:= IncVal(val, 24);
   if met = '4' then nval:= IncVal(val, 12);
   if met = '5' then nval:= DecVal(val, 11);
   if met = '6' then nval:= DecVal(val, 21);
   if met = '7' then nval:= DecVal(val, 22);
   if met = '8' then nval:= DecVal(val, 100);
   if met = '9' then nval:= IncVal(val, 111);
   GetEncCode:= nval;
end;

function GetDecCode(val: byte; p: integer; mask: word): byte;
var
 met, s            : string;
 nval              : byte;
begin
   s:= EPattern[mask];
   met:= s[p+1];
   nval:= val;
   if met = 'A' then nval:= DecVal(val, 2);
   if met = 'B' then nval:= IncVal(val, 5);
   if met = 'C' then nval:= DecVal(val, 9);
   if met = 'D' then nval:= IncVal(val, 3);
   if met = 'E' then nval:= DecVal(val, 10);
   if met = 'F' then nval:= DecVal(val, 44);
   if met = 'G' then nval:= IncVal(val, 33);
   if met = 'H' then nval:= DecVal(val, 11);
   if met = 'I' then nval:= DecVal(val, 19);
   if met = 'J' then nval:= IncVal(val, 8);
   if met = 'K' then nval:= IncVal(val, 45);
   if met = 'L' then nval:= IncVal(val, 32);
   if met = 'M' then nval:= DecVal(val, 77);
   if met = 'N' then nval:= DecVal(val, 66);
   if met = 'O' then nval:= IncVal(val, 55);
   if met = 'P' then nval:= IncVal(val, 1);
   if met = 'Q' then nval:= DecVal(val, 1);
   if met = 'R' then nval:= DecVal(val, 51);
   if met = 'S' then nval:= DecVal(val, 62);
   if met = 'T' then nval:= IncVal(val, 63);
   if met = 'U' then nval:= DecVal(val, 72);
   if met = 'V' then nval:= DecVal(val, 88);
   if met = 'W' then nval:= IncVal(val, 99);
   if met = 'X' then nval:= DecVal(val, 91);
   if met = 'Y' then nval:= DecVal(val, 82);
   if met = 'Z' then nval:= DecVal(val, 85);
   if met = 'a' then nval:= IncVal(val, 92);
   if met = 'b' then nval:= DecVal(val, 94);
   if met = 'c' then nval:= DecVal(val, 96);
   if met = 'd' then nval:= IncVal(val, 83);
   if met = 'e' then nval:= DecVal(val, 38);
   if met = 'f' then nval:= DecVal(val, 37);
   if met = 'g' then nval:= IncVal(val, 36);
   if met = 'h' then nval:= IncVal(val, 24);
   if met = 'i' then nval:= IncVal(val, 25);
   if met = 'j' then nval:= IncVal(val, 26);
   if met = 'k' then nval:= IncVal(val, 27);
   if met = 'l' then nval:= IncVal(val, 87);
   if met = 'm' then nval:= DecVal(val, 8);
   if met = 'n' then nval:= DecVal(val, 9);
   if met = 'o' then nval:= IncVal(val, 13);
   if met = 'p' then nval:= DecVal(val, 16);
   if met = 'q' then nval:= IncVal(val, 19);
   if met = 'r' then nval:= DecVal(val, 21);
   if met = 's' then nval:= DecVal(val, 55);
   if met = 't' then nval:= DecVal(val, 58);
   if met = 'u' then nval:= DecVal(val, 59);
   if met = 'v' then nval:= DecVal(val, 67);
   if met = 'w' then nval:= DecVal(val, 73);
   if met = 'x' then nval:= IncVal(val, 72);
   if met = 'y' then nval:= DecVal(val, 75);
   if met = 'z' then nval:= IncVal(val, 83);
   if met = '0' then nval:= DecVal(val, 41);
   if met = '1' then nval:= DecVal(val, 46);
   if met = '2' then nval:= IncVal(val, 38);
   if met = '3' then nval:= DecVal(val, 24);
   if met = '4' then nval:= DecVal(val, 12);
   if met = '5' then nval:= IncVal(val, 11);
   if met = '6' then nval:= IncVal(val, 21);
   if met = '7' then nval:= IncVal(val, 22);
   if met = '8' then nval:= IncVal(val, 100);
   if met = '9' then nval:= DecVal(val, 111);
   GetDecCode:= nval;
end;



procedure DoEncrypt(len: word; useMask: word);
var
 i                 : integer;
 cnt               : integer;
begin
   if len > 0 then begin
      cnt:= 0;
      for i:= 0 to len - 1 do begin
         eBuf[i]:= GetEncCode(eBuf[i], cnt, maskBuf[useMask]);
         inc(cnt);
         if cnt > 63 then cnt:= 0;
      end;
   end;
end;

procedure DoDecrypt(len: word; useMask: word);
var
 i                 : integer;
 cnt               : integer;
begin
   if len > 0 then begin
      cnt:= 0;
      for i:= 0 to len - 1 do begin
         eBuf[i]:= GetDecCode(eBuf[i], cnt, maskBuf[useMask]);
         inc(cnt);
         if cnt > 63 then cnt:= 0;
      end;
   end;
end;



procedure WriteByte(b: byte);
begin
   write(fff, b);
end;

procedure WriteWord(w: word);
begin
   ValueToBytes(w);
   write(fff, byt[1]);
   write(fff, byt[2]);
end;

procedure WriteString(s: string);
var
 l                 : byte;
 b                 : byte;
 i                 : integer;
begin
   l:= length(s);
   write(fff, l);
   if l > 0 then begin
      for i:= 1 to l do begin
         b:= ord(s[i]);
         write(fff, b);
      end;
   end;
end;

procedure WriteRAWStr(s: string);
var
 l                 : byte;
 b                 : byte;
 i                 : integer;
begin
   l:= length(s);
   if l > 0 then begin
      for i:= 1 to l do begin
         b:= ord(s[i]);
         write(fff, b);
      end;
   end;
end;



function ReadByte: byte;
var
 c                 : byte;
begin
   read(fff, c);
   ReadByte:= c;
end;

function ReadWord: word;
begin
   byt[1]:= ReadByte;
   byt[2]:= ReadByte;
   ReadWord:= BytesToValue(byt[1], byt[2], 0, 0);
end;

function ReadString: string;
var
 l                 : byte;
 i                 : integer;
 c                 : byte;
 s                 : string;
begin
   l:= ReadByte;
   s:= '';
   if l > 0 then begin
      for i:= 1 to l do begin
         c:= ReadByte;
         s:= s + chr(c);
      end;
   end;
   ReadString:= s;
end;

function ReadStringN(ln: integer): string;
var
 i                 : integer;
 s                 : string;
begin
   s:= '';
   if ln > 0 then begin
      for i:= 1 to ln do s:= s + chr(ReadByte);
   end;
   ReadStringN:= s;
end;



begin
   InitMasks;
end.

